@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2012  Kouhei Sutou <kou@clear-code.com>
+# Copyright (C) 2012-2013  Kouhei Sutou <kou@clear-code.com>
 # Copyright (C) 2010  Eddie Lau <tatonlto@gmail.com>
 #
 # License: Ruby's or LGPL
@@ -21,284 +21,9 @@
 require 'gettext/tools/msgmerge'
 
 class TestToolsMsgMerge < Test::Unit::TestCase
-  class TestPoData < self
-    def setup
-      @po_data = GetText::Tools::MsgMerge::PoData.new
-    end
-
-    def test_msgids
-      @po_data["hello"] = "bonjour"
-      @po_data["he"] = "il"
-
-      assert_equal(["he", "hello"], @po_data.msgids.sort)
-    end
-
-    def test_each_msgids
-      @po_data["hello"] = "bonjour"
-      @po_data["he"] = "il"
-      @po_data[""] = "header"
-      @po_data[:last] = ""
-
-      msgids = []
-      @po_data.each_msgid do |msgid|
-        msgids << msgid
-      end
-      assert_equal(["he", "hello"], msgids.sort)
-    end
-
-    class TestAddToExistingEntry < self
-      def test_msgctxt
-        msgctxt = "msgctxt"
-        msgid = "msgid"
-        original_string = "#{msgctxt}\004#{msgid}"
-        msgstr = "msgstr"
-
-        po_data = GetText::Tools::MsgMerge::PoData.new
-        po_data[original_string] = msgstr
-
-        assert_equal(msgctxt, po_data.po[msgctxt, msgid].msgctxt)
-      end
-
-      def test_msgid_plural
-        msgid = "msgid"
-        msgid_plural = "msgid_plural"
-        original_string = "#{msgid}\000#{msgid_plural}"
-        msgstr = "msgstr"
-
-        po_data = GetText::Tools::MsgMerge::PoData.new
-        po_data[msgid] = msgstr
-        assert_equal(nil, po_data.po[msgid].msgid_plural)
-
-        po_data[original_string] = msgstr
-        assert_equal(msgid_plural, po_data.po[msgid].msgid_plural)
-      end
-
-      def test_msgctxt_and_msgid_plural
-        msgctxt = "msgctxt"
-        msgid = "msgid"
-        msgid_plural = "msgid_plural"
-        original_string = "#{msgctxt}\004#{msgid}\000#{msgid_plural}"
-        msgstr = "msgstr"
-
-        po_data = GetText::Tools::MsgMerge::PoData.new
-        po_data[original_string] = msgstr
-
-        assert_equal(msgctxt, po_data.po[msgctxt, msgid].msgctxt)
-        assert_equal(msgid_plural, po_data.po[msgctxt, msgid].msgid_plural)
-      end
-    end
-
-    class TestGeneratePo < self
-      def test_comment_and_msgid_and_msgstr
-        header_entry_comment = "# header entry comment."
-        header_entry = "header\nentry\n"
-        comment = "#: test.rb:10"
-        msgid = "Hello"
-        msgstr = "Salut"
-
-        po = GetText::Tools::MsgMerge::PoData.new
-        po.set_comment("", header_entry_comment)
-        po[""] = header_entry
-        po[msgid] = msgstr
-        po.set_comment(msgid, comment)
-
-        expected_header_entry = ""
-        header_entry.each_line do |line|
-          expected_header_entry << "\"#{line.chomp}\\n\"\n"
-        end
-        expected_header_entry = expected_header_entry.chomp
-        expected_po = <<EOP
-#{header_entry_comment}
-msgid \"\"
-msgstr \"\"
-#{expected_header_entry}
-
-#{comment}
-msgid \"#{msgid}\"
-msgstr \"#{msgstr}\"
-EOP
-        assert_equal(expected_po, po.generate_po)
-      end
-
-      def test_obsolete_comment
-        obsolete_comment =<<EOC
-# test.rb:10
-msgid \"Hello\"
-msgstr \"Salut\"
-EOC
-        header_entry_comment = "# header entry comment."
-        header_entry = "header entry"
-
-        po = GetText::Tools::MsgMerge::PoData.new
-        po.set_comment("", header_entry_comment)
-        po[""] = header_entry
-        po.set_comment(:last, obsolete_comment)
-
-        expected_obsolete_comment = <<-EOC
-# test.rb:10
-#~ msgid "Hello"
-#~ msgstr "Salut"
-EOC
-        expected_obsolete_comment = expected_obsolete_comment.chomp
-
-        expected_po = <<EOP
-#{header_entry_comment}
-msgid \"\"
-msgstr \"#{header_entry}\"
-
-#{expected_obsolete_comment}
-EOP
-        assert_equal(expected_po, po.generate_po)
-      end
-
-      def test_msgid_plural_and_empty_msgstr
-        msgid = "Singular message\000Plural message"
-
-        @po_data[""] = "Plural-Forms: nplurals=2; plural=n != 1;\\n"
-        @po_data[msgid] = "\000"
-        @po_data.set_comment(msgid, "# plural message")
-        actual_po = @po_data.generate_po_entry(msgid)
-        expected_po = <<'EOE'
-# plural message
-msgid "Singular message"
-msgid_plural "Plural message"
-msgstr[0] ""
-msgstr[1] ""
-EOE
-        assert_equal(expected_po, actual_po)
-      end
-    end
-
-    class TestGeneratePOEntry < self
-      def test_msgid_plural
-        msgid = "Singular message\000Plural message"
-
-        @po_data[msgid] = "Singular translation\000Plural translation"
-        @po_data.set_comment(msgid, "# plural message")
-        actual_po = @po_data.generate_po_entry(msgid)
-        expected_po = <<'EOE'
-# plural message
-msgid "Singular message"
-msgid_plural "Plural message"
-msgstr[0] "Singular translation"
-msgstr[1] "Plural translation"
-EOE
-        assert_equal(expected_po, actual_po)
-      end
-
-      def test_msgctxt
-        msg_id = "Context\004Translation"
-        @po_data[msg_id] = "Translated"
-        @po_data.set_comment(msg_id, "# no comment")
-
-        entry = @po_data.generate_po_entry(msg_id)
-        assert_equal(<<-'EOE', entry)
-# no comment
-msgctxt "Context"
-msgid "Translation"
-msgstr "Translated"
-EOE
-      end
-    end
-  end
-
-  class TestSplitMsgid < self
-    def test_existed_msgctxt_and_msgid_plural
-      msgctxt = "msgctxt"
-      msgid = "msgid"
-      msgid_plural = "msgid_plural"
-
-      assert_equal([msgctxt, msgid, msgid_plural],
-                   split_msgid("#{msgctxt}\004#{msgid}\000#{msgid_plural}"))
-    end
-
-    def test_existed_msgctxt_only
-      msgctxt = "msgctxt"
-      msgid = "msgid"
-
-      assert_equal([msgctxt, msgid, nil],
-                   split_msgid("#{msgctxt}\004#{msgid}"))
-    end
-
-    def test_existed_msgid_plural_only
-      msgid = "msgid"
-      msgid_plural = "msgid_plural"
-
-      assert_equal([nil, msgid, msgid_plural],
-                   split_msgid("#{msgid}\000#{msgid_plural}"))
-    end
-
-    def test_not_existed
-      msgid = "msgid"
-
-      assert_equal([nil, msgid, nil], split_msgid(msgid))
-    end
-
-    def test_empty_msgid
-      msgid = ""
-
-      assert_equal([nil, msgid, nil], split_msgid(msgid))
-    end
-
-    def test_last_symbol_msgid
-      msgid = :last
-
-      assert_equal([nil, msgid, nil], split_msgid(msgid))
-    end
-
-    private
-    def split_msgid(msgid)
-      po_data = GetText::Tools::MsgMerge::PoData.new
-      po_data.send(:split_msgid, msgid)
-    end
-  end
-
-  class TestParseComment < self
-    def setup
-      @po_data = GetText::Tools::MsgMerge::PoData.new
-      @entry = @po_data.send(:generate_entry, "test")
-    end
-
-    def test_translator_comment
-      comment = "# translator comment"
-      parsed_comment = parse_comment(comment).translator_comment
-      assert_equal("translator comment\n", parsed_comment)
-    end
-
-    def test_extracted_comment
-      comment = "#. extracted comment"
-      parsed_comment = parse_comment(comment).extracted_comment
-      assert_equal("extracted comment\n", parsed_comment)
-    end
-
-    def test_references
-      comment = "#: reference.rb:10"
-      parsed_comment = parse_comment(comment).references
-      assert_equal(["reference.rb:10"], parsed_comment)
-    end
-
-    def test_flag
-      comment = "#, fuzzy"
-      parsed_comment = parse_comment(comment).flag
-      assert_equal("fuzzy\n", parsed_comment)
-    end
-
-    def test_previous
-      comment = "#| msgid the previous msgid"
-      parsed_comment = parse_comment(comment).previous
-      assert_equal("msgid the previous msgid\n", parsed_comment)
-    end
-
-    private
-    def parse_comment(comment)
-      @po_data.send(:parse_comment, comment, @entry)
-    end
-  end
-
   class TestMerger < self
     def setup
       @merger = GetText::Tools::MsgMerge::Merger.new
-      @po_data = GetText::Tools::MsgMerge::PoData.new
       @po = GetText::PO.new
       @pot = GetText::PO.new
     end
@@ -521,7 +246,7 @@ EOC
     def generate_entry(options)
       msgctxt = options[:msgctxt]
       msgid_plural = options[:msgid_plural]
-      type = @po_data.send(:detect_entry_type, msgctxt, msgid_plural)
+      type = detect_entry_type(msgctxt, msgid_plural)
 
       entry = GetText::POEntry.new(type)
       entry.translator_comment = options[:translator_comment]
@@ -536,9 +261,25 @@ EOC
       entry.comment = options[:comment]
       entry
     end
+
+    def detect_entry_type(msgctxt, msgid_plural)
+      if msgctxt.nil?
+        if msgid_plural.nil?
+          :normal
+        else
+          :plural
+        end
+      else
+        if msgid_plural.nil?
+          :msgctxt
+        else
+          :msgctxt_plural
+        end
+      end
+    end
   end
 
-  class TestMerge < self
+  class TestCommand < self
     include GetTextTestUtils
 
     def setup
@@ -589,12 +330,16 @@ msgstr ""
 "Content-Transfer-Encoding: 8bit\\n"
 "Plural-Forms: nplurals=INTEGER; plural=EXPRESSION;\\n"
 
+#: hello.rb:2
+msgid "World"
+msgstr ""
+
 #: hello.rb:1
 msgid "Hello"
 msgstr ""
 
-#: hello.rb:2
-msgid "World"
+#: hello.rb:3
+msgid "Hello World"
 msgstr ""
 EOP
     end
@@ -632,7 +377,7 @@ EOP
 
     class TestFuzzy < self
       def test_header_message
-        @msgmerge.run(@po_file_path, @pot_file_path, "--output", @po_file_path)
+        @msgmerge.run("--update", @po_file_path, @pot_file_path)
         assert_equal(<<-EOP, File.read(@po_file_path))
 #{po_header(@pot_formatted_time, @po_formatted_time)}
 #: hello.rb:1
@@ -642,8 +387,205 @@ msgstr ""
 #: hello.rb:2
 msgid "World"
 msgstr "Translated World"
+
+#: hello.rb:3
+msgid "Hello World"
+msgstr ""
 EOP
       end
     end
+
+    class TestSort < self
+      def test_default
+        @msgmerge.run("--update", @po_file_path, @pot_file_path)
+        assert_equal(<<-PO, File.read(@po_file_path))
+#{po_header(@pot_formatted_time, @po_formatted_time)}
+#: hello.rb:1
+msgid "Hello"
+msgstr ""
+
+#: hello.rb:2
+msgid "World"
+msgstr "Translated World"
+
+#: hello.rb:3
+msgid "Hello World"
+msgstr ""
+        PO
+      end
+
+      def test_sort_output
+        @msgmerge.run("--update",
+                      "--sort-output",
+                      @po_file_path, @pot_file_path)
+        assert_equal(<<-PO, File.read(@po_file_path))
+#{po_header(@pot_formatted_time, @po_formatted_time)}
+#: hello.rb:1
+msgid "Hello"
+msgstr ""
+
+#: hello.rb:2
+msgid "World"
+msgstr "Translated World"
+
+#: hello.rb:3
+msgid "Hello World"
+msgstr ""
+        PO
+      end
+
+      def test_sort_by_file
+        @msgmerge.run("--update",
+                      "--sort-by-file",
+                      @po_file_path, @pot_file_path)
+        assert_equal(<<-PO, File.read(@po_file_path))
+#{po_header(@pot_formatted_time, @po_formatted_time)}
+#: hello.rb:1
+msgid "Hello"
+msgstr ""
+
+#: hello.rb:2
+msgid "World"
+msgstr "Translated World"
+
+#: hello.rb:3
+msgid "Hello World"
+msgstr ""
+        PO
+      end
+
+      def test_sort_by_msgid
+        @msgmerge.run("--update",
+                      "--sort-by-msgid",
+                      @po_file_path, @pot_file_path)
+        assert_equal(<<-PO, File.read(@po_file_path))
+#{po_header(@pot_formatted_time, @po_formatted_time)}
+#: hello.rb:1
+msgid "Hello"
+msgstr ""
+
+#: hello.rb:3
+msgid "Hello World"
+msgstr ""
+
+#: hello.rb:2
+msgid "World"
+msgstr "Translated World"
+        PO
+      end
+    end
+
+    class TestLocation < self
+      def test_location
+        @msgmerge.run("--update",
+                      "--location",
+                      @po_file_path, @pot_file_path)
+        assert_equal(<<-PO, File.read(@po_file_path))
+#{po_header(@pot_formatted_time, @po_formatted_time)}
+#: hello.rb:1
+msgid "Hello"
+msgstr ""
+
+#: hello.rb:2
+msgid "World"
+msgstr "Translated World"
+
+#: hello.rb:3
+msgid "Hello World"
+msgstr ""
+        PO
+      end
+
+      def test_no_location
+        @msgmerge.run("--update",
+                      "--no-location",
+                      @po_file_path, @pot_file_path)
+        assert_equal(<<-PO, File.read(@po_file_path))
+#{po_header(@pot_formatted_time, @po_formatted_time)}
+msgid "Hello"
+msgstr ""
+
+msgid "World"
+msgstr "Translated World"
+
+msgid "Hello World"
+msgstr ""
+        PO
+      end
+    end
+
+    class TestWidth < self
+      def pot_content
+        <<-POT
+#: hello.rb:1
+msgid "Hello very long line! This line is very long. Yes! This line is very long! Very very long line!"
+msgstr ""
+
+#: hello.rb:3
+msgid "Hello World"
+msgstr ""
+        POT
+      end
+
+      def po_content
+        <<-PO
+#: hello.rb:3
+msgid "Hello World"
+msgstr "Translated Hello World. This translation is very long. Yes! Very long translation!!!"
+        PO
+      end
+
+      def test_default
+        @msgmerge.run("--update",
+                      @po_file_path, @pot_file_path)
+        assert_equal(<<-PO, File.read(@po_file_path))
+#: hello.rb:1
+msgid ""
+"Hello very long line! This line is very long. Yes! This line is very long! Ver"
+"y very long line!"
+msgstr ""
+
+#: hello.rb:3
+msgid "Hello World"
+msgstr ""
+"Translated Hello World. This translation is very long. Yes! Very long translat"
+"ion!!!"
+        PO
+      end
+
+      def test_width
+        @msgmerge.run("--update",
+                      "--width", "70",
+                      @po_file_path, @pot_file_path)
+        assert_equal(<<-PO, File.read(@po_file_path))
+#: hello.rb:1
+msgid ""
+"Hello very long line! This line is very long. Yes! This line is very l"
+"ong! Very very long line!"
+msgstr ""
+
+#: hello.rb:3
+msgid "Hello World"
+msgstr ""
+"Translated Hello World. This translation is very long. Yes! Very long "
+"translation!!!"
+        PO
+      end
+
+      def test_no_wrap
+        @msgmerge.run("--update",
+                      "--no-wrap",
+                      @po_file_path, @pot_file_path)
+        assert_equal(<<-PO, File.read(@po_file_path))
+#: hello.rb:1
+msgid "Hello very long line! This line is very long. Yes! This line is very long! Very very long line!"
+msgstr ""
+
+#: hello.rb:3
+msgid "Hello World"
+msgstr "Translated Hello World. This translation is very long. Yes! Very long translation!!!"
+        PO
+      end
+    end
   end
 end
